package cd.casic.ci.common.pipeline;

import cd.casic.ci.common.pipeline.container.Container;
import cd.casic.ci.common.pipeline.container.Stage;
import cd.casic.ci.common.pipeline.container.TriggerContainer;
import cd.casic.ci.common.pipeline.enums.CallBackEvent;
import cd.casic.ci.common.pipeline.pojo.event.PipelineCallbackEvent;
import cd.casic.ci.common.pipeline.pojo.event.ProjectPipelineCallBack;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import cd.casic.ci.common.pipeline.pojo.transfer.Resources;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline
 * @Project：ops-pro
 * @name：Model
 * @Date：2025/03/25 17:39
 * @Filename：Model
 * @description：Todo
 */
@Data
@Schema(title = "流水线模型-创建信息")
public class Model implements IModelTemplate {
    @Schema(title = "名称", required = true)
    private String name;

    @Schema(title = "描述", required = false)
    private String desc;

    @Schema(title = "阶段集合", required = true)
    private List<Stage> stages;

    @Schema(title = "标签", required = false)
    @Deprecated
    //"以PipelineGroupService和流水线设置版本中为准"
    private List<String> labels;

    @Schema(title = "是否从模板中实例化出来的", required = false)
    private Boolean instanceFromTemplate;

    @Schema(title = "创建人", required = false)
    private String pipelineCreator;

    @Schema(title = "当前模板对应的被复制的模板或安装的研发商店的模板对应的ID", required = false)
    private String srcTemplateId;

    @Schema(title = "当前模板的ID", required = false)
    private String templateId;

    @Schema(title = "提示", required = false)
    private String tips;

    @Schema(title = "流水线事件回调", required = false)
    private Map<String, PipelineCallbackEvent> events;

    @Schema(title = "静态流水线组", required = false)
    private List<String> staticViews;

    @Schema(title = "各项耗时", required = true)
    private BuildRecordTimeCost timeCost;

    @Schema(title = "模板地址", required = true)
    private String template;

    @Schema(title = "模板版本", required = true)
    private String ref;

    @Schema(title = "模板入参", required = true)
    private Map<String, String> variables;

    @Schema(title = "模板资源", required = true)
    private Resources resources;

    @Schema(title = "提交时流水线最新版本号", required = false)
    private int latestVersion;
    /**
     * 获取流水线回调配置
     * @param projectId 项目ID
     * @param callbackEvent 回调事件类型
     * @return 流水线回调配置列表
     */
    public  List<ProjectPipelineCallBack> getPipelineCallBack(
            String projectId,
            CallBackEvent callbackEvent
    ) {
        List<ProjectPipelineCallBack> pipelineCallBack = new ArrayList<>();
        if (events != null) {
            events.forEach((eventName, event) -> {
                // TODO 可能要重写equals
                if (callbackEvent==event.getCallbackEvent()) {
                    pipelineCallBack.add(
                            new ProjectPipelineCallBack(
                                    null,
                                    projectId,
                                    event.getCallbackEvent().name(),
                                    event.getCallbackUrl(),
                                    event.getSecretToken(),
                                    true,
                                    null,
                                    null,
                                    null
                            )
                    );
                }
            });
        }
        return pipelineCallBack;
    }
    public TriggerContainer getTriggerContainer(){
        Container container = stages.get(0).getContainers().get(0);
        return JSON.parseObject(JSON.toJSONString(container),TriggerContainer.class);
    }
}
