package cd.casic.ci.log.api.impl;

import cd.casic.ci.log.dal.pojo.LogLine;
import cd.casic.ci.log.dal.pojo.QueryLogStatus;
import cd.casic.ci.log.dal.pojo.QueryLogs;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.log.api.UserLogResource;
import cd.casic.ci.log.dal.pojo.enums.LogType;
import cd.casic.ci.log.service.BuildLogQueryService;
import io.micrometer.core.annotation.Timed;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


@RestController
public class UserLogResourceImpl implements UserLogResource {
    @Resource
    private BuildLogQueryService buildLogQueryService;
    @Resource
    private MeterRegistry meterRegistry;

    private static final int defaultNum = 100;

    @Value("${spring.application.name}")
    private String applicationName = "";

    @Override
    @Timed
    public CommonResult<QueryLogs> getInitLogs(String userId, String projectId, String pipelineId,
                                               String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        CommonResult<QueryLogs> initLogs = buildLogQueryService.getInitLogs(userId, projectId, pipelineId, buildId, debug, logType, tag, jobId, executeCount, subTag);
        List<LogLine> logs = initLogs.getData().getLogs();
        if (logs == null) {
            recordListLogCount(0);
        } else {
            recordListLogCount(logs.size());
        }
        return initLogs;
    }

    @Override
    public CommonResult<QueryLogs> getMoreLogs(String userId, String projectId, String pipelineId,
                                               String buildId, boolean debug, LogType logType, int num, boolean fromStart, long start, long end, String tag, String subTag, String jobId, int executeCount) {
        return buildLogQueryService.getMoreLogs(userId, projectId, pipelineId, buildId, debug, logType, num, fromStart, start, end, tag, jobId, executeCount, subTag);
    }

    @Override
    public CommonResult<QueryLogs> getAfterLogs(String userId, String projectId, String pipelineId, String buildId, long start, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        CommonResult<QueryLogs> afterLogs = buildLogQueryService.getAfterLogs(userId, projectId,
                pipelineId, buildId, start, debug, logType, tag, jobId, executeCount, subTag);
        List<LogLine> logs = afterLogs.getData().getLogs();
        if (logs == null) {
            recordListLogCount(0);
        } else {
            recordListLogCount(logs.size());
        }
        return afterLogs;
    }

    @Override
    public ResponseEntity downloadLogs(String userId, String projectId, String pipelineId, String buildId, String tag, String subTag, String jobId, int executeCount, String fileName) {
        return buildLogQueryService.downloadLogs(userId, projectId, pipelineId, buildId, tag, jobId, executeCount, fileName, subTag);
    }

    @Override
    public CommonResult<QueryLogStatus> getLogMode(String userId, String projectId, String pipelineId, String buildId, String tag, int executeCount) {
        return buildLogQueryService.getLogMode(userId, projectId, pipelineId, buildId, tag, executeCount);
    }

    private void recordListLogCount(int count) {
        Counter.builder("list_log_count")
                .tag("application", applicationName)
                .register(meterRegistry)
                .increment(count);
    }
}
