package cd.casic.ci.process.process.service.pipeline.impl;

import cd.casic.ci.common.pipeline.req.pipeline.PipelineCreateReq;
import cd.casic.ci.process.process.dal.pipeline.PipelineDao;
import cd.casic.ci.process.process.dataObject.pipeline.PipPipeline;
import cd.casic.ci.process.process.dataObject.resource.PipResource;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.pipeline.PipelineService;
import cd.casic.ci.process.process.service.resource.impl.ResourceServiceImpl;
import cd.casic.ci.process.process.service.stage.impl.StageServiceImpl;
import cd.casic.ci.process.process.service.task.impl.TaskServiceImpl;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.framework.commons.util.util.WebFrameworkUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName PipelineServiceImpl
 * @Date: 2025/5/13 10:31
 * @Description:
 */
@Service
@Slf4j
public class PipelineServiceImpl extends ServiceImpl<PipelineDao, PipPipeline> implements PipelineService {
    @Resource
    private StageServiceImpl stageService;

    @Resource
    private TaskServiceImpl taskService;

    @Resource
    private ResourceServiceImpl resourceService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createPipeline(PipelineCreateReq pipelineReq) {

        PipPipeline pipeline = new PipPipeline();
        BeanUtils.copyProperties(pipelineReq, pipeline);
        // 随机颜色
        int randomNumber = (int)(Math.random() * 5) + 1;
        pipeline.setColor(randomNumber);

        pipeline.setCreateTime(LocalDateTime.now());
        if (Objects.isNull(pipeline.getUserId())){
            String loginUserId = String.valueOf(WebFrameworkUtils.getLoginUserId());
            pipeline.setUserId(loginUserId);
        }

        pipeline.setState(1);
        this.save(pipeline);

        //TODO 根据模板覆写对应的阶段信息以及节点信息
//        List<StageReq> stageReqList = templateService.findStageById(pipelineReq.getTemplateId());
//        stageReqList.forEach(o->{
//            o.setPipelineId(pipeline.getId());
//        });
        //新增初始阶段，无串行并行任务
        PipStage stageReq1 = new PipStage();
        stageReq1.setPipelineId(pipeline.getId());
        stageReq1.setStageName("阶段-1");
        stageReq1.setCreateTime(LocalDateTime.now());
        stageReq1.setStageSort(1);
        stageReq1.setParentId("-1");
        stageReq1.setCode(true);
        stageService.save(stageReq1);

        PipStage childStage1 = new PipStage();
        childStage1.setStageName("源码");
        childStage1.setCreateTime(LocalDateTime.now());
        childStage1.setStageSort(1);
        childStage1.setCode(false);
        childStage1.setParentId(stageReq1.getStageId());
        stageService.save(childStage1);

        //新增第二个阶段，包含串行和并行任务
        PipStage stageReq2 = new PipStage();
        stageReq2.setPipelineId(pipeline.getId());
        stageReq2.setStageName("阶段-2");
        stageReq2.setCreateTime(LocalDateTime.now());
        stageReq2.setStageSort(2);
        stageReq2.setParentId("-1");
        stageReq2.setCode(true);
        stageService.save(stageReq2);

        //新增并行阶段
        PipStage childStage21 = new PipStage();
        childStage21.setStageName("并行阶段-2-1");
        childStage21.setCreateTime(LocalDateTime.now());
        childStage21.setStageSort(1);
        childStage21.setCode(false);
        childStage21.setParentId(stageReq2.getStageId());
        stageService.save(childStage21);

        //新增串行阶段
        PipTask childTask21 = new PipTask();
        childTask21.setCreateTime(LocalDateTime.now());
        childTask21.setTaskName("串行任务-2-1");
        childTask21.setTaskType("test");
        childTask21.setTaskSort(1);
        childTask21.setStageId(childStage21.getStageId());
        taskService.save(childTask21);

        //------------------------------------------------------------------
        PipStage childStage22 = new PipStage();
        childStage22.setStageName("并行阶段-2-2");
        childStage22.setCreateTime(LocalDateTime.now());
        childStage22.setStageSort(2);
        childStage22.setCode(false);
        childStage22.setParentId(stageReq2.getStageId());
        stageService.save(childStage22);

        PipTask childTask22 = new PipTask();
        childTask22.setCreateTime(LocalDateTime.now());
        childTask22.setTaskName("串行任务-2-2");
        childTask22.setTaskType("test");
        childTask22.setTaskSort(2);
        childTask22.setStageId(childStage22.getStageId());
        taskService.save(childTask22);

        //创建第三个阶段
        PipStage stageReq3 = new PipStage();
        stageReq3.setPipelineId(pipeline.getId());
        stageReq3.setStageName("阶段-3");
        stageReq3.setCreateTime(LocalDateTime.now());
        stageReq3.setStageSort(3);
        stageReq3.setParentId("-1");
        stageReq3.setCode(true);
        stageService.save(stageReq3);

        PipStage childStage3 = new PipStage();
        childStage3.setStageName("");
        childStage3.setCreateTime(LocalDateTime.now());
        childStage3.setStageSort(1);
        childStage3.setCode(false);
        childStage3.setParentId(stageReq3.getStageId());
        stageService.save(childStage3);

        PipTask childTask31 = new PipTask();
        childTask31.setCreateTime(LocalDateTime.now());
        childTask31.setTaskName("串行任务-3-1");
        childTask31.setTaskType("test");
        childTask31.setTaskSort(1);
        childTask31.setStageId(childStage3.getStageId());
        taskService.save(childTask31);

        //创建第四个阶段
        PipStage stageReq4 = new PipStage();
        stageReq4.setPipelineId(pipeline.getId());
        stageReq4.setStageName("阶段-4");
        stageReq4.setCreateTime(LocalDateTime.now());
        stageReq4.setStageSort(4);
        stageReq4.setParentId("-1");
        stageReq4.setCode(true);
        stageService.save(stageReq4);

        PipStage childStage4 = new PipStage();
        childStage4.setStageName("");
        childStage4.setCreateTime(LocalDateTime.now());
        childStage4.setStageSort(1);
        childStage4.setCode(false);
        childStage4.setParentId(stageReq4.getStageId());
        stageService.save(childStage4);

        PipTask childTask41 = new PipTask();
        childTask41.setCreateTime(LocalDateTime.now());
        childTask41.setTaskName("串行任务-4-1");
        childTask41.setTaskType("test");
        childTask41.setTaskSort(1);
        childTask41.setStageId(childStage4.getStageId());
        taskService.save(childTask41);

        //TODO 创建对应的鉴权关系
        //TODO 创建对应的消息分发

        //新增对应的配置类数据
        if (!ObjectUtils.isEmpty(pipelineReq.getResourceReq())){
            PipResource resource = new PipResource();
            BeanUtils.copyProperties(pipelineReq.getResourceReq(),resource);
            resource.setPipelineId(pipeline.getId());
            resourceService.save(resource);
        }
        return pipeline.getId();
    }
}
