package cd.casic.ci.process.process.service.stage.impl;


import cd.casic.ci.common.pipeline.req.stage.StageCreateReq;
import cd.casic.ci.common.pipeline.req.stage.StageReq;
import cd.casic.ci.common.pipeline.req.task.TaskCreateReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;

import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.util.object.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static cd.casic.ci.process.constant.PipelineFinalConstant.TASK_TYPE_CODE;

@Service
public class StageServiceImpl extends ServiceImpl<PipStageDao, PipStage> implements StageService {
    @Resource
    private TaskService taskService;
    @Resource
    private PipStageDao stageDao;
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String createStagesOrTask(StageCreateReq stageReq) {
        String firstStageId = stageReq.getStageId();
        Integer stageSort = stageReq.getStageSort();
        TaskCreateReq task = stageReq.getTask();
        String taskName = task.getTaskName();
        String secondStageId = task.getStageId();
        JSONObject taskProperties = task.getTaskProperties();
        String taskType = task.getTaskType();
        Integer taskSort = task.getTaskSort();
        String pipelineId = task.getPipelineId();
        PipStage firstStage = null;
        // 判断是否需要新建阶段,如果没有传第一级tageId而传了sort就是需要创建阶段
        if (StringUtils.isEmpty(firstStageId)) {
            // 新建stage
            firstStage = new PipStage();
            firstStage.setStageName("阶段-"+stageSort);
            if (stageSort == null) {
                throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"参数有误");
            }
            firstStage.setStageSort(stageSort);
            firstStage.setCreateTime(LocalDateTime.now());
            firstStage.setParentId("-1");
            firstStage.setPipelineId(pipelineId);
            PipStage stageQuery = new PipStage();
            stageQuery.setPipelineId(pipelineId);
            stageQuery.setParentId("-1");
            List<PipStage> otherStageList = getPipStageList(stageQuery);
            if (!CollectionUtils.isEmpty(otherStageList)) {
                for (PipStage stage : otherStageList) {
                    if (stageSort<=stage.getStageSort()) {
                        stage.setStageSort(stage.getStageSort()+1);
                    }
                }
                updateBatchById(otherStageList);
            }
            save(firstStage);
//            firstStage
        } else {
            PipStage stageQuery = new PipStage();
            stageQuery.setStageId(firstStageId);
            List<PipStage> pipStageList = getPipStageList(stageQuery);
            if (!CollectionUtils.isEmpty(pipStageList)) {
                firstStage = pipStageList.get(0);
            }
        }
        if (firstStage == null) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"创建节点失败");
        }
        PipStage secondStage = null;
        if (secondStageId==null||taskSort==null) {
            // 添加并行节点
            secondStage = new PipStage();
            PipStage stageQuery = new PipStage();
            stageQuery.setParentId(firstStageId);
            stageQuery.setPipelineId(pipelineId);
            List<PipStage> pipStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(pipStageList)) {
                secondStage.setStageSort(1);
            } else {
                secondStage.setStageSort(pipStageList.size()+1);
            }
            secondStage.setCreateTime(LocalDateTime.now());
            save(secondStage);
            taskSort=1;
            secondStageId = secondStage.getStageId();
        } else {
            secondStage = getById(secondStageId);
        }
        if (secondStage == null) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"创建节点失败");
        }
        // 保存task
        PipTask pipTask = new PipTask();
        pipTask.setTaskType(taskType);
        pipTask.setTaskSort(taskSort);
        pipTask.setStageId(secondStageId);
        pipTask.setTaskName(taskName);
        pipTask.setCreateTime(LocalDateTime.now());
        pipTask.setTaskProperties(taskProperties);
        pipTask.setPipelineId(pipelineId);
        taskService.save(pipTask);
        return pipTask.getTaskId();
    }
    /**
     * 更新主节点阶段顺序
     * @param pipelineId 流水线id
     * @param taskSort 顺序
     * @return 顺序
     */
    private Integer initStage(String pipelineId,int taskSort){
        List<PipStage> allMainStage = findAllMainStage(pipelineId);
        if (allMainStage.isEmpty()){
            return 1;
        }
        for (PipStage stages : allMainStage) {
            int stage = stages.getStageSort();
            if (stage < taskSort){
                continue;
            }
            stages.setStageSort(stage+1);
            updateStages(stages);
        }
        return taskSort;
    }
    @Override
    public void createStageTemplate(String pipelineId, String[] template) {

    }

    @Override
    public void cloneStage(String pipelineId, String clonePipelineId) {

    }

    @Override
    public List<StageResp> findAllStagesOrTask(String pipelineId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesTask(String pipelineId) {
        //获取流水线主节点
        List<PipStage> stageMainStage = findAllMainStage(pipelineId);
        if (stageMainStage.isEmpty()){
            return Collections.emptyList();
        }
        List<StageResp> list = new ArrayList<>();
        for (PipStage stage : stageMainStage) {
            String stagesId = stage.getStageId();
            //获取从节点
            List<PipStage> allStageStage = findOtherStageNoTask(stagesId);
            List<StageResp> stageRespList = allStageStage.stream().map(it -> {
                StageResp stageResp = new StageResp();
                BeanUtils.copyProperties(stageResp, it);
                return stageResp;
            }).toList();
            StageResp stageResp = new StageResp();
            BeanUtils.copyProperties(stage,stageResp);
            stageResp.setStageList(stageRespList);
            list.add(stageResp);
        }
        list.sort(Comparator.comparing(StageResp::getStageSort));
        return list;
    }

    @Override
    public void deleteStagesOrTask(String taskId) {
        PipTask taskQuery = new PipTask();
        taskQuery.setTaskId(taskId);
        List<PipTask> taskList = taskService.getTask(taskQuery);
        if (CollectionUtils.isEmpty(taskList)) {
            return;
        }
        PipTask task = taskList.get(0);
        taskService.deleteTaskById(taskId);
        // 查询上一级stage下有无其他task 没有则连着stage删除
        String stageId = task.getStageId();
        String pipelineId = task.getPipelineId();
        taskQuery.setTaskId("");
        taskQuery.setStageId(stageId);
        List<PipTask> otherTask = taskService.getTask(task);
        if (CollectionUtils.isEmpty(otherTask)) {
            // 删除当前task的父stage，然后判断父stage的父级有无其他子集如果没有就继续删除当前阶段

            PipStage stageQuery = new PipStage();
            stageQuery.setPipelineId(stageId);
            List<PipStage> currStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(currStageList)) {
                throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"stage不存在");
            }
            PipStage currStage = currStageList.get(0);
            deleteStages(stageId);
            String parentId = currStage.getParentId();
            stageQuery.setParentId(parentId);
            stageQuery.setStageId(null);
            // 查询同阶段其他二级stage，如果不存在其他stage则删除阶段stage并整理sort值
            List<PipStage> otherStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(otherStageList)) {
                //没有其他并行路径就需要删除当前阶段
                deleteStages(parentId);
            } else {
                for (PipStage stage : otherStageList) {
                    if (currStage.getStageSort()<stage.getStageSort()) {
                        stage.setStageSort(stage.getStageSort()-1);
                    }
                }
            }
        } else {
            for (PipTask pipTask : otherTask) {
                if (task.getTaskSort()<pipTask.getTaskSort()) {
                    pipTask.setTaskSort(pipTask.getTaskSort()-1);
                }
            }
        }

    }

    @Override
    public void deleteAllStagesOrTask(String pipelineId) {

    }

    private List<PipStage> getPipStageList(PipStage pipStage){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getStageId()),PipStage::getStageId,pipStage.getStageId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getPipelineId()),PipStage::getPipelineId,pipStage.getPipelineId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getParentId()),PipStage::getParentId,pipStage.getParentId());
        return stageDao.selectList(wrapper);
    }

    @Override
    public void updateStageName(StageReq stage) {
        LambdaUpdateWrapper<PipStage> wrapper = new LambdaUpdateWrapper<>();
        if (StringUtils.isEmpty(stage.getStageName())||StringUtils.isEmpty(stage.getStageId())) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"阶段Id和名称蹦年为空");
        }
        wrapper.eq(PipStage::getStageId,stage.getStageId());
        wrapper.set(PipStage::getStageName,stage.getStageName());
        stageDao.update(wrapper);
    }

    @Override
    public List<PipStage> findAllMainStage(String pipelineId) {
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getPipelineId,pipelineId);
        wrapper.eq(PipStage::getParentId,"-1");
        return stageDao.selectList(wrapper);
    }

    @Override
    public void updateStagesTask(StageReq stage) {

    }

    @Override
    public List<String> validStagesMustField(String pipelineId) {
        return null;
    }

    @Override
    public String createStages(PipStage stage) {
        stageDao.insert(stage);
        return stage.getStageId();
    }

    @Override
    public void updateStages(PipStage stage) {
        stageDao.updateById(stage);
    }

    @Override
    public void deleteStages(String stageId) {
        stageDao.deleteById(stageId);
    }

    @Override
    public PipStage findOneStages(String stageId) {
        return null;
    }

    @Override
    public List<StageResp> findAllStages() {
        return null;
    }

    @Override
    public List<StageResp> findAllStagesList(List<String> idList) {
        return null;
    }

    @Override
    public void createStagesOrTaskList(List<StageCreateReq> stageList) {
        if (!ObjectUtils.isEmpty(stageList)){
            stageList.forEach(this::createStagesOrTask);
        }
    }

    /**
     * 判断任务是否存在代码源
     * @param pipelineId 流水线id
     */
    private void findTargetTasks(String pipelineId) {
        List<PipStage> allStage = findAllMainStage(pipelineId);
        if (allStage.isEmpty()){
            return;
        }
        for (PipStage stage : allStage) {
            if (stage.getCode()){
                throw new ServiceException(50001,"代码源已存在，无法再次创建。");
            }
        }
    }
    public List<PipStage> findOtherStageNoTask(String stagesId){
        List<PipStage> otherStage = findSecondStage(stagesId);
        List<PipStage> list = new ArrayList<>();
        for (PipStage stage : otherStage) {
            //获取阶段配置及任务
            String otherId = stage.getStageId();
            List<PipTask> allStageTask = taskService.finAllStageTask(otherId);
            stage.setTaskValues(allStageTask);
            list.add(stage);
        }
        return list;
    }
    // 获取主stage（阶段）下的子stage
    @Override
    public List<PipStage> findSecondStage(String stageId){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getParentId,stageId);
        return stageDao.selectList(wrapper);
    }
}
