package cd.casic.ci.process.process.service.task.impl;

import cd.casic.ci.common.pipeline.annotation.Plugin;
import cd.casic.ci.common.pipeline.resp.task.TasksResp;
import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dal.pipeline.PipTaskDao;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.exception.ServiceException;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cd.casic.ci.process.constant.PipelineFinalConstant.TASK_TYPE_CODE;

@Service
public class TaskServiceImpl extends ServiceImpl<PipTaskDao, PipTask> implements TaskService {
    private static final String basePackage = "cd.casic.ci.process.process.service";
    private Set<BeanDefinition> candidates;
    @Resource
    private PipTaskDao taskDao;
    @Resource
    private PipStageDao stageDao;

    public TaskServiceImpl() {
        System.out.println("");
    }

    @PostConstruct
    public void init(){
        ClassPathScanningCandidateComponentProvider provider =
                new ClassPathScanningCandidateComponentProvider(false);
        provider.addIncludeFilter(new AnnotationTypeFilter(Plugin.class));
        candidates = provider.findCandidateComponents(basePackage);
    }
    @Override
    public void taskTypeExist(String taskType) {

    }

    @Override
    public String createTasksOrTask(PipTask tasks)  {

        int sort = 0;
        int taskSort = tasks.getTaskSort();
        String taskType = tasks.getTaskType();

        //判断多任务是否存在代码源
        boolean b = TASK_TYPE_CODE.equals(taskType);
        if (tasks.getPipelineId() != null && (b) ){
            findCode(tasks.getPipelineId());
        }

        //流水线任务
        String pipelineId = tasks.getPipelineId();
        if (pipelineId != null) {
            sort = initSort(tasks.getPipelineId(), taskSort, taskType,1);
        }

        //阶段任务
        String stageId = tasks.getStageId();
        if (stageId != null){
            sort = initSort(stageId, taskSort, taskType,2);
        }
        // TODO 后置任务暂时空置
//        //后置任务
//        String postprocessId = tasks.getPostprocessId();
//        if (postprocessId != null){
//            sort = initSort(postprocessId, taskSort, taskType,3);
//        }

        tasks.setTaskSort(sort);
        // 初始化名称
        if (Objects.isNull(tasks.getTaskName())){
//            String taskName = initDifferentTaskName(taskType);
            String taskName = "未命名";
            tasks.setTaskName(taskName);
        }

        tasks.setCreateTime(LocalDateTime.now());

        String tasksId = createTasks(tasks);
        //创建任务
//        createDifferentTask(tasksId,taskType,tasks.getValues());
        return tasksId;
    }

    @Override
    public List<PipTask> finAllStageTask(String stageId) {
        LambdaQueryWrapper<PipTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipTask::getStageId,stageId);
        return taskDao.selectList(wrapper);
    }

    @Override
    public List<PipTask> getTask(PipTask task) {
        LambdaQueryWrapper<PipTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(task.getTaskId()),PipTask::getTaskId,task.getTaskId());
        wrapper.eq(StringUtils.isNotEmpty(task.getPipelineId()),PipTask::getPipelineId,task.getPipelineId());
        wrapper.eq(StringUtils.isNotEmpty(task.getStageId()),PipTask::getStageId,task.getStageId());
        return taskDao.selectList(wrapper);
    }
    @Override
    public TasksResp getById(String taskId){
        PipTask pipTask = new PipTask();
        pipTask.setTaskId(taskId);
        List<PipTask> taskList = getTask(pipTask);
        if (!CollectionUtils.isEmpty(taskList)) {
            return null;
        }
        TasksResp tasksResp = new TasksResp();
        BeanUtils.copyProperties(taskList.get(0),tasksResp);
        return tasksResp;
    }
    @Override
    public void deleteTaskById(String taskId) {
        taskDao.deleteById(taskId);
    }

    void createDifferentTask(String taskId,String taskType,Object values){
        // TODO
    }
    /**
     *  查找目标节点是否存在
     * */
    private void findCode(String pipelineId){
        List<PipTask> tasks = finAllPipelineTask(pipelineId);
        if (tasks == null){
            return;
        }
        for (PipTask task : tasks) {
            String taskType = task.getTaskType();
            if (TASK_TYPE_CODE.equals(taskType)){
                throw new ServiceException(50001,"代码源已存在");
            }
        }
    }

    private List<PipTask> finAllPipelineTask(String pipelineId) {
        LambdaQueryWrapper<PipTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipTask::getPipelineId,pipelineId);
        return taskDao.selectList(wrapper);
    }
    /**
     * 初始化配置顺序
     * @param id id
     * @param taskSort 插入顺序
     * @param taskType 任务类型
     * @param type 1.流水线id 2.阶段id
     * @return 顺序
     */
    private Integer initSort(String id, int taskSort,String taskType,int type){
        List<PipTask> list = new ArrayList<>();
        if (type == 1){
            list = finAllPipelineTask(id);
        }
        if (type == 2){
            list = findAllStageTask(id);
        }
//        if (type == 3){
//            PipTask postTask = findOnePostTask(id);
//            if (postTask!= null){
//                list.add(postTask);
//            }
//        }

        if (list.isEmpty()){
            return 1;
        }

        boolean b = TASK_TYPE_CODE.equals(taskType);

        //插入的为代码源
        if (b){
            for (PipTask tasks : list) {
                tasks.setTaskSort(tasks.getTaskSort()+1);
                updateTasks(tasks);
            }
            return 1;
        }

        //更新顺序
        for (PipTask tasks : list) {
            if (tasks.getTaskSort() < taskSort ){
                continue;
            }
            tasks.setTaskSort(tasks.getTaskSort()+1);
            updateTasks(tasks);
        }
        return taskSort;

    }

    private void updateTasks(PipTask tasks){
        taskDao.updateById(tasks);
    }
    /**
     * 创建任务
     * @param tasks 任务模型
     * @return 任务id
     */
    private String createTasks(PipTask tasks){
        taskDao.insert(tasks);
        return tasks.getTaskId();
    }
    /**
     *  根据stageId查询task
     * */
    private List<PipTask> findAllStageTask(String stageId) {
//        TasksQuery tasksQuery = new TasksQuery();
//        tasksQuery.setStageId(stageId);
//        List<TasksEntity> tasksEntityList = tasksDao.findTaskList(tasksQuery);
//        return BeanMapper.mapList(tasksEntityList, Tasks.class);
        LambdaQueryWrapper<PipTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipTask::getStageId,stageId);
        return taskDao.selectList(wrapper);
    }
}
