package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.media.MediaInfoReq;
import cd.casic.pipeline.store.pojo.common.media.StoreMediaInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "OP_COMMON_MEDIA", description = "研发商店_媒体信息")
@RestController
@RequestMapping("/op/store/media")
public interface OpMediaResource {

    @Operation(summary = "提交媒体信息")
    @PostMapping("/storeCodes/{storeCode}/types/{storeType}/media")
    CommonResult<Boolean> createStoreMedia(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "研发商店代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "类别", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "评论信息", required = true)
            List<MediaInfoReq> mediaInfoList
    );

    @Operation(summary = "获取单条媒体信息")
    @GetMapping("/ids/{mediaId")
    CommonResult<StoreMediaInfo> getStoreMedia(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "媒体ID", required = true)
            @PathVariable("mediaId")
            String mediaId
    );

    @Operation(summary = "获取扩展服务所有媒体信息")
    @GetMapping("/storesCodes/{storeCode}/types/{labelType}")
    CommonResult<List<StoreMediaInfo>> getStoreMediaByStoreCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "研发商店编码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "类别", required = true)
            @PathVariable("labelType")
            StoreTypeEnum labelType
    );
}