package cd.casic.pipeline.store.api.common;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.ReasonTypeEnum;
import cd.casic.pipeline.store.pojo.common.reason.Reason;
import cd.casic.pipeline.store.pojo.common.reason.ReasonReq;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "OP_STORE_REASON", description = "OP-STORE-原因")
@RestController
@RequestMapping("/op/store/reason")
public interface OpReasonResource {

    @Operation(summary = "添加原因")
    @PostMapping("/types/{type}")
    CommonResult<Boolean> add(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "原因类型", required = true)
            @PathVariable("type")
            ReasonTypeEnum type,
            @Parameter(description = "原因信息请求报文体", required = true)
            ReasonReq reasonReq
    );

    @Operation(summary = "更新原因信息")
    @PutMapping("/types/{type}/ids/{id}")
    CommonResult<Boolean> update(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "id", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "类别", required = true)
            @PathVariable("type")
            ReasonTypeEnum type,
            @Parameter(description = "原因信息请求报文体", required = true)
            ReasonReq reasonReq
    );

    @Operation(summary = "启用禁用原因")
    @PutMapping("/types/{type}/ids/{id}/enable")
    CommonResult<Boolean> enableReason(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "id", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "类别", required = true)
            @PathVariable("type")
            ReasonTypeEnum type,
            @Parameter(description = "是否启用", required = true)
            Boolean enable
    );

    @Operation(summary = "获取原因列表")
    @GetMapping("/types/{type}/list")
    CommonResult<List<Reason>> list(
            @Parameter(description = "类别", required = true)
            @PathVariable("type")
            ReasonTypeEnum type,
            @Parameter(description = "是否启用", required = false)
            @RequestParam("enable")
            Boolean enable
    );

    @Operation(summary = "删除原因")
    @DeleteMapping("/types/{type}/ids/{id}")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "id", required = true)
            @PathVariable("id")
            String id
    );
}