package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveConfReq;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveConfResp;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_MARKET_SENSITIVECONF", description = "敏感数据配置")
@RestController
@RequestMapping("/user/market/{storeType}/component/{storeCode}/sensitiveConf/")
public interface UserSensitiveConfResource {

    @Operation(summary = "插件工作台-新增敏感数据")
    @PostMapping("/")
    CommonResult<Boolean> create(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "插件工作台-新增敏感数据报文体", required = true)
            SensitiveConfReq sensitiveConfReq
    );

    @Operation(summary = "插件工作台-编辑敏感数据")
    @PutMapping("/{id}")
    CommonResult<Boolean> edit(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "敏感配置ID", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "插件工作台-编辑敏感数据报文体", required = true)
            SensitiveConfReq sensitiveConfReq
    );

    @Operation(summary = "删除敏感数据")
    @DeleteMapping("/")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "敏感数据ID集合，用\",\"分隔进行拼接（如1,2,3）", required = true)
            @RequestParam("ids")
            String ids
    );

    @Operation(summary = "获取敏感数据")
    @GetMapping("/{id}")
    CommonResult<SensitiveConfResp> get(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "ID", required = true)
            @PathVariable("id")
            String id
    );

    @Operation(summary = "获取敏感数据")
    @GetMapping("/list")
    CommonResult<List<SensitiveConfResp>> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "字段类型集合，用\",\"分隔进行拼接（如BACKEND,FRONTEND,ALL）", required = false)
            @RequestParam("types")
            String types
    );
}