package cd.casic.server.controller;

import cd.casic.ci.common.pipeline.req.stage.StageCreateReq;
import cd.casic.ci.common.pipeline.req.stage.StageReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.framework.commons.pojo.CommonResult;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;

import jakarta.annotation.Resource;
import jakarta.annotation.security.PermitAll;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;

import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;


import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @pi.protocol: http
 * @pi.groupName: 流水线多阶段控制器
 */
@RestController
@RequestMapping("/stage")
public class StageController {

    @Resource
    private StageService stageService;

    /**
     * @pi.name:创建流水线阶段及任务
     * @pi.path:/stage/createStage
     * @pi.methodType:post
     * @pi.request-type:json
     * @pi.param: model=stage
     */
    @RequestMapping(path="/createStage",method = RequestMethod.POST)
    public CommonResult<String> createStagesOrTask(@RequestBody @Valid @NotNull StageCreateReq stage){
        String taskId = stageService.createStagesOrTask(stage);
        return CommonResult.success(taskId);
    }


    @RequestMapping(path="/createStageList",method = RequestMethod.POST)
    public CommonResult<String> createStagesOrTaskList(@RequestBody @Valid @NotNull List<StageCreateReq> stageList){
        stageService.createStagesOrTaskList(stageList);
        return CommonResult.success();
    }

    /**
     * @pi.name:查询流水线阶段信息
     * @pi.path:/stage/finAllStage
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=pipelineId;dataType=string;value=pipelineId;
     */
    @PermitAll
    @RequestMapping(path="/finAllStage",method = RequestMethod.POST)
    public CommonResult<List<StageResp>> finAllPipelineTaskOrTask(@NotNull String pipelineId){
        List<StageResp> stageRespList = stageService.findAllStagesTask(pipelineId);
        return CommonResult.success(stageRespList);
    }

    /**
     * // TODO 暂时没发现哪里调用
     * @pi.name:更新流水线阶段任务
     * @pi.path:/stage/createStage
     * @pi.methodType:post
     * @pi.request-type:json
     * @pi.param: model=stage
     */
    @RequestMapping(path="/updateStage",method = RequestMethod.POST)
    public CommonResult<Void> updateStageTask(@RequestBody @Valid @NotNull StageReq stage){
        stageService.updateStagesTask(stage);
        return CommonResult.success();
    }

    /**
     * @pi.name:更新流水线阶段名称
     * @pi.path:/stage/updateStageName
     * @pi.methodType:post
     * @pi.request-type:json
     * @pi.param: model=stage
     */
    @PermitAll
    @RequestMapping(path="/updateStageName",method = RequestMethod.POST)
    public CommonResult<Void> updateTasksStage(@RequestBody @Valid @NotNull StageReq stage){
        stageService.updateStageName(stage);
        return CommonResult.success();
    }

    /**
     * @pi.name:删除流水线阶段及任务
     * @pi.path:/stage/deleteStage
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=taskId;dataType=string;value=taskId;
     */
    @RequestMapping(path="/deleteStage",method = RequestMethod.POST)
    public CommonResult<Void> deleteTasksOrStage(@NotNull String taskId){
        stageService.deleteStagesOrTask(taskId);
        return CommonResult.success();
    }

    /**
     * @pi.name:效验流水线各个配置阶段完整性
     * @pi.path:/stage/validStagesMustField
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=pipelineId;dataType=string;value=流水线Id;
     */
    @RequestMapping(path="/validStagesMustField",method = RequestMethod.POST)
    public CommonResult<List<String>> validStagesMustField(@NotNull String pipelineId){
//        List<String> list = stageService.validStagesMustField(pipelineId);
//        return CommonResult.ok(list);
        return CommonResult.success(Collections.emptyList());
    }



}




























